.jouvence-doc {
  font-family: "Courier Prime", "Courier New", Courier, sans-serif;
  background-color: #fff;
  color: #111; }
  .jouvence-doc hr {
    border: none;
    height: 1px;
    background: #aaa; }
  .jouvence-doc .jouvence-main {
    padding: 2em 0; }
    .jouvence-doc .jouvence-main p {
      padding: 0 2em; }
  .jouvence-doc h1, .jouvence-doc .jouvence-title-page-heading {
    text-align: center; }
  .jouvence-doc .jouvence-scene-heading {
    font-weight: bold; }
  .jouvence-doc .jouvence-character {
    margin: auto 12rem 0 12rem; }
  .jouvence-doc .jouvence-parenthetical {
    margin: 0 8rem 0 8rem; }
  .jouvence-doc .jouvence-dialog {
    margin: 0 4rem 0 4rem; }
  .jouvence-doc .jouvence-action-centered {
    text-align: center; }
  .jouvence-doc p.jouvence-section {
    background-color: #ccc;
    padding: 1rem; }
  .jouvence-doc p.jouvence-synopsis {
    background-color: #ddd;
    padding: 1rem; }
  .jouvence-doc .jouvence-notes {
    background-color: #593;
    color: #fff;
    padding: 1em 0; }
    .jouvence-doc .jouvence-notes p {
      padding: 0 2em; }

/*# sourceMappingURL=html_styles.css.map */
